package com.laihua.projecthelper.implement

import java.io.File

/**
 * Author: pengjunjing
 * Date: 2022/3/7
 * Description:
 */
class FileCalc(val file: File) {


    /**
     * 去除后缀的名字,代码依赖的时候,不需要使用后缀的
     */
    val implementName: String

    //依赖的代码
    val implementCode = mutableListOf<CodeFileFindResult>()

    val belongModule = this.file.belongModule()

    init {
        //文件名 ,带后缀,去除后缀
        val name = file.name
        implementName = if (name.contains(".")) {
            name.substring(0, name.indexOf("."))
        } else {
            name
        }
    }

    val implementModule: MutableSet<String> = mutableSetOf()

    /**
     * 统计该资源被哪些模块使用了
     */
    fun calcImplementModule() {
        implementModule.clear()
        implementCode.forEach {
            it.belongModule?.let { moduleName ->
                implementModule.add(moduleName)
            }
        }
    }


}

/**
 * @param file 文件
 * @param codeLine 调用的代码
 */
class CodeFileFindResult(val file: File, val codeLine: List<String>) {

    val belongModule = this.file.belongModule()

}


/**
 * 判断一个文件是属于哪个模块的
 * 返回的大小写不确定
 */
fun File.belongModule(): String? {
    return this.belongModuleFile()?.name
}

/**
 * 从一个文件开始,向上查找,直到查找到所在模块
 */
fun File.belongModuleFile(): File? {
    var start = this
    while (start.parentFile != null) {
        if (hasModuleDir(start)) {
            return start
        }
        start = start.parentFile
    }
    return null
}

/**
 * 判断一个目录是否为模块的根目录
 */
fun hasModuleDir(file: File): Boolean {
    val listFiles = file.listFiles()
    if (!listFiles.isNullOrEmpty()) {
        for (listFile in listFiles) {
            if (listFile.name.lowercase() == "build.gradle") {
                return true
            }
        }
    }
    return false
}
