package com.laihua.projecthelper.stringxml

import com.laihua.projecthelper.code.*
import com.laihua.projecthelper.unuse.ReadUtil
import java.io.File

/**
 * Author: pengjunjing
 * Date: 2022/3/10
 * Description:
 */
class StringPull {
}

fun main(args: Array<String>) {

    println("开始读取文件")
    val projectRoot = File("")
    val projectPath = projectRoot.absolutePath
    val projectRootF = File(projectPath)
    //这里打印的应该会是as项目所处的路径
    println(projectPath)
    //要检查资源的目录,大小写敏感
    val baseFile =
        File("$projectPath${File.separator}LaiHuaBase${File.separator}src${File.separator}")
    //从项目根目录开始查找
//    val baseFile = File("${projectRoot.absolutePath}")
    println(baseFile.absolutePath + "\n")
    //读取代码文件
    val readCode = ProjectCode().readCode(StringRefCodeReader())
    //读取string配置的文件
    val readStringResFile = ReadUtil().readStringResFile(baseFile)
    readStringResFile.forEach {
        println("读取到的字符串资源文件:${it.absolutePath}")
    }

    //解析代码,得到依赖的结果
    val stringXmlResReferList: List<StringXmlResCode> = readStringResFile.map {
        parseStringResFile(it, readCode)
    }

    //未使用的string定义
    for (stringXmlResCode in stringXmlResReferList) {
        val filter = stringXmlResCode.stringElementList.filter {
            it.refer.isEmpty()
        }
        println("未被使用的字符串一共有${filter.size}个")
        filter.forEach {
            println("未被使用的字符串 id:${it.attributeName} 内容是: ${it.value}")
            //删除未使用的string资源
            StringXmlParse().deleteStringXmlElement(it)
        }
    }

    println("\n--------------------------------\n")

    for (stringXmlResCode in stringXmlResReferList) {
        val filter = stringXmlResCode.stringElementList.filter {
            //过滤出只有一个模块依赖的string
            it.refer.size == 1
        }
        println("${stringXmlResCode.stringXmlFile.absolutePath} 被一个模块依赖的字符串有:${filter.size}个")
        filter.forEach { stringElement ->
            val module = stringElement.refer.toList()[0]
            println("模块 ${module.moduleName} 依赖了字符串:${stringElement.attributeName} , 内容是:  ${stringElement.value}")
            StringXmlParse().moveStringXmlElement(stringElement, module)
        }
    }

    println("\n--------------------------------\n")


}


/**
 * 解析一个string.xml文件,判断有多少个模块依赖
 * @return 返回解析出来的依赖结果
 */
private fun parseStringResFile(file: File, readCode: MutableSet<Module>): StringXmlResCode {
    //将一个xml文件解析成每个节点的值
    val stringXmlResCode: StringXmlResCode = StringXmlParse().parseToObj(file)
    //遍历xml文件的每个字符串
    everyStringDef@ for (stringElement in stringXmlResCode.stringElementList) {
        //遍历每个模块
        everyModule@ for (module: Module in readCode) {
            //遍历模块下的每个代码文件
            everyCodeFile@ for (code: Code in module.codes) {
                val javaKtXmlCode = code as JavaKtXmlCode
                //某个代码文件的每行有调用string资源的代码
                everyCode@ for (codeLine: String in javaKtXmlCode.codeList) {
                    if (
                    //代码中包含:   "@string/tips_bind_phone"
                        codeLine.contains("\"@string/${stringElement.attributeName}\"")
                        //代码中包含:   R.string.xxx
                        || codeLine.contains("R.string.${stringElement.attributeName}")
                    ) {
                        //添加到引用模块去
                        stringElement.refer.add(module)
                        //该字符串被该模块引用了则直接跳过该模块后面的代码判断了
                        break@everyCodeFile
                    }
                }
            }
        }

    }
    return stringXmlResCode
}

