package com.laihua.projecthelper.unuse

import java.io.File

/**
 * Author: pengjunjing
 * Date: 2022/3/4
 * Description:
 */
class ReadUtil {

    //读取图片资源
    fun readImageRes(file: File): MutableList<File> {
        val resResult = mutableListOf<File>()
        resResult.clear()
        if (file.exists() && file.isDirectory) {
            readChildFile(file, ".gif", ".png", ".jpg", ".webp", ".9", resultList = resResult)
        } else {
            println("资源目录不存在或者传入的不是目录:${file.absolutePath}")
        }
        return resResult
    }


    /**
     * 读取代码文件
     */
    fun readCodeFile(file: File): MutableList<File> {
        val codeResult = mutableListOf<File>()
        codeResult.clear()
        if (file.exists() && file.isDirectory) {
            readChildFile(file, ".java", ".kt", ".xml", resultList = codeResult)
        } else {
            println("资源目录不存在或者传入的不是目录:${file.absolutePath}")
        }
        return codeResult
    }

    val valuesDirName = "values"
    private val excludeXml = arrayOf("values", "AndroidManifest", "res/xml", "res\\xml")

    /**
     * 读取xml图片文件
     */
    fun readXmlFile(file: File): List<File> {
        val result = mutableListOf<File>()
        if (file.exists() && file.isDirectory) {
            readChildFile(file, ".xml", resultList = result)
        } else {
            println("资源目录不存在或者传入的不是目录:${file.absolutePath}")
        }

        return result.filter {
            var filterResult = true
            for (exclude in excludeXml) {
                //排除掉部分xml文件
                if (it.absolutePath.contains(exclude, ignoreCase = true)) {
                    filterResult = false
                    break
                }
            }
            filterResult
        }
    }

    /**
     * 读取values文件夹下的文件
     */
    private fun readValueFile(file: File): List<File> {
        val result = mutableListOf<File>()
        if (file.exists() && file.isDirectory) {
            readChildFile(file, ".xml", resultList = result)
        } else {
            println("资源目录不存在或者传入的不是目录:${file.absolutePath}")
        }

        return result.filter {
            //只需要values文件夹下的文件
            it.absolutePath.contains(valuesDirName, ignoreCase = true)
        }
    }

    /**
     * 读取资源目录下的string文件
     * @param file 指定目录
     */
    fun readStringResFile(file: File): List<File> {
        val readXmlFile = readValueFile(file)

        val filter = readXmlFile.filter { xmlFile ->
            val readLines = xmlFile.readLines()
            //是否包含resources定义
            var containResString = false
            var containString = false
            //读取文件每一行
            for (readLine in readLines) {
                //判断是否包含 resources 和 string的标签
                if (readLine.contains(resourcesNode)) {
                    containResString = true
                }
                if (readLine.contains(StringNode)) {
                    containString = true
                }
                if (containResString && containString) {
                    //是定义字符串的资源文件
                    return@filter true
                }
            }
            return@filter false
        }
        return filter
    }

    companion object {
        //排除目录
        private val excludeDir = arrayOf("build", ".git", ".gradle", ".idea")
        const val resourcesNode = "<resources>"
        const val StringNode = "<string"

        /**
         * @param fileType 文件后缀名
         */
        private fun readChildFile(file: File, vararg fileType: String, resultList: MutableList<File>) {
            val listFiles = file.listFiles()
            if (listFiles != null && listFiles.isNotEmpty()) {
                //该目录下存在文件or文件夹
                out@ for (it in listFiles) {
                    if (it.isDirectory) {
                        for (s in excludeDir) {
                            if (it.name.equals(s)) {
                                //排除指定目录
                                continue@out
                            }
                        }
                        //目录递归遍历
                        readChildFile(it, *fileType, resultList = resultList)
                    } else if (it.isFile) {
                        for (s in fileType) {
                            if (it.name.lowercase().endsWith(s.lowercase())) {
                                //文件后缀名匹配
                                resultList.add(it)
                            }
                        }
                    }
                }
            }
        }
    }


}